{-# LANGUAGE ImportQualifiedPost #-}
{-# LANGUAGE OverloadedStrings #-}

module Main (main) where

import Data.ByteString.Char8 qualified as B
import Data.Text.IO qualified as T
import Main.LLVMGen
import Main.Parser.Megaparsec
import Options.Applicative
import System.Environment
import System.Process (callCommand)

data Opt = Opt
  { filePath :: String,
    showLLVM :: Bool,
    showDebug :: Bool
  }

run :: Opt -> IO ()
run opts = do
  let fileName = filePath opts
  contents <- T.readFile fileName
  T.putStrLn "- Generating LLVM to './a.out.ll'..."
  result <- (llvmGen . parse) contents
  B.writeFile "a.out.ll" result
  T.putStrLn "- Compiling to executable './a.out'..."
  callCommand "clang a.out.ll"
  T.putStrLn "- Done."

main :: IO ()
main = execParser opts >>= run
  where
    parser =
      Opt
        <$> argument str (metavar "FILE_PATH")
        <*> switch
          ( short 'l'
              <> long "showLLVM"
              <> help "Create <file>.ll with LLVM used to compile the binary"
          )
        <*> switch
          ( short 'd'
              <> long "showDebug"
              <> help "Show debug output"
          )
    opts = info parser mempty
