-- see https://markkarpov.com/tutorial/megaparsec.html
{-# LANGUAGE ImportQualifiedPost #-}
{-# LANGUAGE OverloadedStrings #-}

module Main.Parser.Megaparsec (parse) where

import Control.Monad.Combinators.Expr
import Data.Functor.Identity qualified
import Data.Text
import Data.Void (Void)
import Main.Type
import Text.Megaparsec as MP hiding (parse)
import Text.Megaparsec qualified as MP
import Text.Megaparsec.Char qualified as C
import Text.Megaparsec.Char.Lexer qualified as L

type Parser = Parsec Void Text

lexeme :: Parser a -> Parser a
lexeme = L.lexeme C.space

symbol :: Text -> Parser Text
symbol = L.symbol C.space

int :: Parser Int
int = lexeme $ L.signed (return ()) L.decimal

string :: Text -> Parser Text
string = C.string

container :: Text -> Text -> Parser a -> Parser a
container b e = between (symbol b) (symbol e)

term :: Parser Expr
term =
  choice
    [ Lit <$> int,
      container "(" ")" expr
    ]

table :: [[Operator Parser Expr]]
table =
  [ [methodOp "print" Print],
    [ binaryOp "*" Mul,
      binaryOp "/" Div
    ],
    [ binaryOp "+" Add,
      binaryOp "-" Sub
    ]
  ]

binaryOp :: Text -> (a -> a -> a) -> Operator (ParsecT Void Text Data.Functor.Identity.Identity) a
binaryOp name f = InfixL $ f <$ symbol name

-- prefixOp :: Text -> (a -> a) -> Operator (ParsecT Void Text Data.Functor.Identity.Identity) a
-- prefixOp name f = Prefix (f <$ symbol name)

methodOp :: Text -> (a -> a) -> Operator (ParsecT Void Text Data.Functor.Identity.Identity) a
methodOp name f = Prefix $ f <$ (string name <* C.space)

-- postfixOp :: Text -> (a -> a) -> Operator (ParsecT Void Text Data.Functor.Identity.Identity) a
-- postfixOp name f = Postfix (f <$ symbol name)

expr :: Parser Expr
expr = makeExprParser term table

statement :: Parser Expr
statement = expr <* symbol ";"

parseExpr :: Text -> Either (ParseErrorBundle Text Void) [Expr]
parseExpr = MP.parse (C.space *> many statement <* eof) ""

parse :: Text -> [Expr]
parse t =
  case parseExpr t of
    Right r -> r

-- TODO: add error handling